@@ -224,11 +224,23 @@ int ovc, ovr;
 	/* Apply oversize. */
 	ov_cols = 0;
 	ov_rows = 0;
-	if (ovc * ovr >= 0x4000)
-		xs_warning("Illegal %s, ignoring", ResOversize);
-	else if (ovc >= maxCOLS && ovr >= maxROWS) {
-		ov_cols = maxCOLS = COLS = ovc;
-		ov_rows = maxROWS = ROWS = ovr;
+	if (ovc != 0 || ovr != 0) {
+		if (ovc <= 0 || ovr <= 0)
+			popup_an_error("Invalid %s %dx%d:\nNegative or zero",
+			    ResOversize, ovc, ovr);
+		else if (ovc * ovr >= 0x4000)
+			popup_an_error("Invalid %s %dx%d:\nToo big",
+			    ResOversize, ovc, ovr);
+		else if (ovc > 0 && ovc < maxCOLS)
+			popup_an_error("Invalid %s cols (%d):\nLess than model %d cols (%d)",
+			    ResOversize, ovc, model_num, maxCOLS);
+		else if (ovr > 0 && ovr < maxROWS)
+			popup_an_error("Invalid %s rows (%d):\nLess than model %d rows (%d)",
+			    ResOversize, ovr, model_num, maxROWS);
+		else {
+			ov_cols = maxCOLS = COLS = ovc;
+			ov_rows = maxROWS = ROWS = ovr;
+		}
 	}
 
 	/* Update the model name. */
@@ -508,10 +520,11 @@ Boolean *anyp;
 }
 
 static void
-insert_sa(baddr, current_fgp, current_grp, anyp)
+insert_sa(baddr, current_fgp, current_grp, current_csp, anyp)
 int baddr;
 unsigned char *current_fgp;
 unsigned char *current_grp;
+unsigned char *current_csp;
 Boolean *anyp;
 {
 	if (reply_mode != SF_SRM_CHAR)
@@ -527,6 +540,15 @@ Boolean *anyp;
 			gr |= 0xf0;
 		insert_sa1(XA_HIGHLIGHTING, gr, current_grp, anyp);
 	}
+	if (memchr((char *)crm_attr, XA_CHARSET, crm_nattr)) {
+		unsigned char cs;
+
+		cs = ea_buf[baddr].cs & CS_MASK;
+		if (cs)
+			cs |= 0xf0;
+		insert_sa1(XA_CHARSET, cs, current_csp, anyp);
+	}
+
 }
 
 
@@ -544,6 +566,7 @@ Boolean all;
 	Boolean		short_read = False;
 	unsigned char	current_fg = 0x00;
 	unsigned char	current_gr = 0x00;
+	unsigned char	current_cs = 0x00;
 
 	trace_ds("> ");
 	obptr = obuf;
@@ -604,9 +627,11 @@ Boolean all;
 					if (send_data &&
 					    screen_buf[baddr]) {
 						insert_sa(baddr,
-						    &current_fg, &current_gr,
+						    &current_fg,
+						    &current_gr,
+						    &current_cs,
 						    &any);
-						if (ea_buf[baddr].cs) {
+						if (ea_buf[baddr].cs & CS_GE) {
 							space3270out(1);
 							*obptr++ = ORDER_GE;
 							if (any)
@@ -638,8 +663,11 @@ Boolean all;
 		do {
 			if (screen_buf[baddr]) {
 				insert_sa(baddr,
-				    &current_fg, &current_gr, &any);
-				if (ea_buf[baddr].cs) {
+				    &current_fg,
+				    &current_gr,
+				    &current_cs,
+				    &any);
+				if (ea_buf[baddr].cs & CS_GE) {
 					space3270out(1);
 					*obptr++ = ORDER_GE;
 					if (any)
@@ -698,6 +726,7 @@ unsigned char aid_byte;
 	int		attr_count;
 	unsigned char	current_fg = 0x00;
 	unsigned char	current_gr = 0x00;
+	unsigned char	current_cs = 0x00;
 
 	trace_ds("> ");
 	obptr = obuf;
@@ -744,11 +773,24 @@ unsigned char aid_byte;
 					    ea_buf[baddr].gr | 0xf0));
 					(*(obuf + attr_count))++;
 				}
+				if (ea_buf[baddr].cs & CS_MASK) {
+					space3270out(2);
+					*obptr++ = XA_CHARSET;
+					*obptr++ =
+					    (ea_buf[baddr].cs & CS_MASK) | 0xf0;
+					trace_ds("%s", see_efa(XA_CHARSET,
+					    (ea_buf[baddr].cs & CS_MASK) | 0xf0));
+					(*(obuf + attr_count))++;
+				}
 			}
 			any = False;
 		} else {
-			insert_sa(baddr, &current_fg, &current_gr, &any);
-			if (ea_buf[baddr].cs) {
+			insert_sa(baddr,
+			    &current_fg,
+			    &current_gr,
+			    &current_cs,
+			    &any);
+			if (ea_buf[baddr].cs & CS_GE) {
 				space3270out(1);
 				*obptr++ = ORDER_GE;
 				if (any)
@@ -791,6 +833,7 @@ ctlr_snap_buffer()
 	int		attr_count;
 	unsigned char	current_fg = 0x00;
 	unsigned char	current_gr = 0x00;
+	unsigned char	current_cs = 0x00;
 	unsigned char   av;
 
 	obptr = obuf;
@@ -818,6 +861,12 @@ ctlr_snap_buffer()
 				*obptr++ = ea_buf[baddr].gr | 0xf0;
 				(*(obuf + attr_count))++;
 			}
+			if (ea_buf[baddr].cs & CS_MASK) {
+				space3270out(2);
+				*obptr++ = XA_CHARSET;
+				*obptr++ = (ea_buf[baddr].cs & CS_MASK) | 0xf0;
+				(*(obuf + attr_count))++;
+			}
 		} else {
 			av = ea_buf[baddr].fg;
 			if (current_fg != av) {
@@ -837,7 +886,17 @@ ctlr_snap_buffer()
 				*obptr++ = XA_HIGHLIGHTING;
 				*obptr++ = av;
 			}
-			if (ea_buf[baddr].cs) {
+			av = ea_buf[baddr].cs & CS_MASK;
+			if (av)
+				av |= 0xf0;
+			if (current_cs != av) {
+				current_cs = av;
+				space3270out(3);
+				*obptr++ = ORDER_SA;
+				*obptr++ = XA_CHARSET;
+				*obptr++ = av;
+			}
+			if (ea_buf[baddr].cs & CS_GE) {
 				space3270out(1);
 				*obptr++ = ORDER_GE;
 			}
@@ -957,12 +1016,19 @@ Boolean erase;
 	int		any_fa;
 	unsigned char	efa_fg;
 	unsigned char	efa_gr;
+	unsigned char	efa_cs;
 	char		*paren = "(";
 	enum { NONE, ORDER, SBA, TEXT, NULLCH } previous = NONE;
 
 #define END_TEXT0	{ if (previous == TEXT) trace_ds("'"); }
 #define END_TEXT(cmd)	{ END_TEXT0; trace_ds(" %s", cmd); }
 
+#define ATTR2FA(attr) \
+	(FA_BASE | \
+	 (((attr) & 0x20) ? FA_PROTECT : 0) | \
+	 (((attr) & 0x10) ? FA_NUMERIC : 0) | \
+	 (((attr) & 0x01) ? FA_MODIFY : 0) | \
+	 (((attr) >> 2) & FA_INTENSITY))
 #define START_FIELDx(fa) { \
 			current_fa = &(screen_buf[buffer_addr]); \
 			ctlr_add(buffer_addr, fa, 0); \
@@ -973,14 +1039,7 @@ Boolean erase;
 		}
 #define START_FIELD0	{ START_FIELDx(FA_BASE); }
 #define START_FIELD(attr) { \
-			new_attr = FA_BASE; \
-			if ((attr) & 0x20) \
-				new_attr |= FA_PROTECT; \
-			if ((attr) & 0x10) \
-				new_attr |= FA_NUMERIC; \
-			if ((attr) & 0x01) \
-				new_attr |= FA_MODIFY; \
-			new_attr |= ((attr) >> 2) & FA_INTENSITY; \
+			new_attr = ATTR2FA(attr); \
 			START_FIELDx(new_attr); \
 		}
 
@@ -1072,6 +1131,22 @@ Boolean erase;
 		case ORDER_PT:	/* program tab */
 			END_TEXT("ProgramTab");
 			previous = ORDER;
+			/*
+			 * If the buffer address is the field attribute of
+			 * of an unprotected field, simply advance one
+			 * position.
+			 */
+			if (IS_FA(screen_buf[buffer_addr]) &&
+			    !FA_IS_PROTECTED(screen_buf[buffer_addr])) {
+				INC_BA(buffer_addr);
+				last_zpt = False;
+				last_cmd = True;
+				break;
+			}
+			/*
+			 * Otherwise, advance to the first position of the
+			 * next unprotected field.
+			 */
 			baddr = next_unprotected(buffer_addr);
 			if (baddr < buffer_addr)
 				baddr = 0;
@@ -1090,6 +1165,7 @@ Boolean erase;
 				}
 				if (baddr == 0)
 					last_zpt = True;
+
 			} else
 				last_zpt = False;
 			buffer_addr = baddr;
@@ -1118,7 +1194,10 @@ Boolean erase;
 				return;
 			}
 			do {
-				if (ra_ge || default_cs)
+				if (ra_ge)
+					ctlr_add(buffer_addr, ebc2cg0[*cp],
+					    CS_GE);
+				else if (default_cs)
 					ctlr_add(buffer_addr, ebc2cg0[*cp], 1);
 				else
 					ctlr_add(buffer_addr, ebc2cg[*cp], 0);
@@ -1162,7 +1241,7 @@ Boolean erase;
 			trace_ds(see_ebc(*cp));
 			if (*cp)
 				trace_ds("'");
-			ctlr_add(buffer_addr, ebc2cg0[*cp], 1);
+			ctlr_add(buffer_addr, ebc2cg0[*cp], CS_GE);
 			ctlr_add_fg(buffer_addr, default_fg);
 			ctlr_add_gr(buffer_addr, default_gr);
 			INC_BA(buffer_addr);
@@ -1186,7 +1265,11 @@ Boolean erase;
 						if (*cp == XA_3270) {
 							trace_ds(" 3270");
 							cp++;
-							START_FIELD(*cp);
+							new_attr = ATTR2FA(*cp);
+							ctlr_add(buffer_addr,
+							    new_attr,
+							    0);
+							trace_ds(see_attr(new_attr));
 						} else if (*cp == XA_FOREGROUND) {
 							trace_ds("%s",
 							    see_efa(*cp,
@@ -1200,6 +1283,17 @@ Boolean erase;
 								*(cp + 1)));
 							cp++;
 							ctlr_add_gr(buffer_addr, *cp & 0x07);
+						} else if (*cp == XA_CHARSET) {
+							int cs = 0;
+
+							trace_ds("%s",
+							    see_efa(*cp,
+								*(cp + 1)));
+							cp++;
+							if (*cp == 0xf1)
+								cs = 1;
+							ctlr_add(buffer_addr,
+							    screen_buf[buffer_addr], cs);
 						} else if (*cp == XA_ALL) {
 							trace_ds("%s",
 							    see_efa(*cp,
@@ -1227,6 +1321,7 @@ Boolean erase;
 			any_fa = 0;
 			efa_fg = 0;
 			efa_gr = 0;
+			efa_cs = 0;
 			for (i = 0; i < (int)na; i++) {
 				cp++;
 				if (*cp == XA_3270) {
@@ -1243,6 +1338,11 @@ Boolean erase;
 					trace_ds("%s", see_efa(*cp, *(cp + 1)));
 					cp++;
 					efa_gr = *cp & 0x07;
+				} else if (*cp == XA_CHARSET) {
+					trace_ds("%s", see_efa(*cp, *(cp + 1)));
+					cp++;
+					if (*cp == 0xf1)
+						efa_cs = 1;
 				} else if (*cp == XA_ALL) {
 					trace_ds("%s", see_efa(*cp, *(cp + 1)));
 					cp++;
@@ -1253,6 +1353,7 @@ Boolean erase;
 			}
 			if (!any_fa)
 				START_FIELD0;
+			ctlr_add(buffer_addr, screen_buf[buffer_addr], efa_cs);
 			ctlr_add_fg(buffer_addr, efa_fg);
 			ctlr_add_gr(buffer_addr, efa_gr);
 			INC_BA(buffer_addr);
@@ -1274,6 +1375,10 @@ Boolean erase;
 				trace_ds("%s", see_efa(*cp, *(cp + 1)));
 				default_fg = 0;
 				default_gr = 0;
+				default_cs = 0;
+			} else if (*cp == XA_CHARSET) {
+				trace_ds("%s", see_efa(*cp, *(cp + 1)));
+				default_cs = (*(cp + 1) == 0xf1) ? 1 : 0;
 			} else
 				trace_ds("%s[unsupported]",
 				    see_efa(*cp, *(cp + 1)));
