@@ -67,6 +67,7 @@
 #define ResMenuBar		"menuBar"
 #define ResModel		"model"
 #define ResModifiedSel		"modifiedSel"
+#define ResModifiedSelColor	"modifiedSelColor"
 #define ResMono			"mono"
 #define ResMonoCase		"monoCase"
 #define ResNoOther		"noOther"
@@ -173,6 +174,7 @@
 #define ClsMenuBar		"MenuBar"
 #define ClsModel		"Model"
 #define ClsModifiedSel		"ModifiedSel"
+#define ClsModifiedSelColor	"ModifiedSelColor"
 #define ClsMono			"Mono"
 #define ClsMonoCase		"MonoCase"
 #define ClsNoOther		"NoOther"
